% 'Fiscal Activism and the Zero Nominal Interest Rate Bound'
% Sebastian Schmidt, sebastian.schmidt@ecb.int

% Grid search for optimal degree of fiscal activism
% Model with ZLB and natural rate shock, and partial price indexation

% Subroutines called:
% NKMsolve
% NKMsimul
% NKMirf_d
% CompEcon toolbox: fundefn, funnode, qnwnorm, funeval

clear all;
clc;
close all;
format long;

addpath('D:\sebastian\COMPeconToolbox\CEtools');
% addpath('\\GIMECB01\HOMEDIR-QU$\schmids\Research Sebastian\COMPeconToolbox\CEtools');

maxit = 50000;

% Parameters
beta = 0.995; 
rstar = 100*(1/beta-1);
GY = 0.2;
alpha = 0.784;
theta = 13.23;
sigma = 1.22;
omega = sigma*(1-GY)/GY;  
eta = 1.69;
tau = 0.3; % DEGREE OF PRICE INDEXATION
Gamma = sigma/(sigma+eta);
kappa = (1-beta*alpha)*(1-alpha)/alpha*(sigma+eta)/(1+eta*theta);
lambda = kappa/theta; 
lambdaGstar = lambda*Gamma*(1-Gamma+omega/sigma);
rho = 0.8; 
stdevd = 0.2975; 
uncovad = stdevd^2/(1-rho^2);
uncostdevd = sqrt(uncovad);


params.rstar = rstar;
params.beta = beta;
params.sigma = sigma;
params.eta = eta;
params.Gamma = Gamma;
params.kappa = kappa;
params.lambda = lambda;
params.lambdaGstar = lambdaGstar;
params.rho = rho;
params.uncostdevd = uncostdevd;
params.stdevd = stdevd;
params.tau = tau;


% Collocation
% State variables: natural rate, lagged inflation rate

smin   = [-uncostdevd*4 -0.5];         
smax   = [uncostdevd*4 0];
fspace = fundefn('spli',[45 25],smin,smax);

tol = 1.49*10^(-8); % tolerance crition for iteration

scoord = funnode(fspace); % collocation nodes
snodes = gridmake(scoord);
Phi = funbas(fspace,snodes);

upd = 0.2; %0.2; % Updating parameter to avoid overshooting
up = 0.1; % 0.1; 

% Gaussian Quadrature 
[epsilons,ws] = qnwnorm(7,0, stdevd^2); 

lambdaGvec = (0.00005:0.00005:0.002)';
loss = zeros(length(lambdaGvec),1);
Closs = zeros(length(lambdaGvec),1);
cpvec = zeros(length(snodes),length(lambdaGvec));
cyvec = zeros(length(snodes),length(lambdaGvec));
cgvec = zeros(length(snodes),length(lambdaGvec));
civec = zeros(length(snodes),length(lambdaGvec));
Ep_pvec = zeros(length(snodes),length(lambdaGvec));
Ey_pvec = zeros(length(snodes),length(lambdaGvec));
Eg_pvec = zeros(length(snodes),length(lambdaGvec));
zlbfreq = zeros(length(lambdaGvec),1);
sd_pi = zeros(length(lambdaGvec),1);
sd_y = zeros(length(lambdaGvec),1);
sd_g = zeros(length(lambdaGvec),1);
sd_i = zeros(length(lambdaGvec),1);
m_pi = zeros(length(lambdaGvec),1);
m_y = zeros(length(lambdaGvec),1);
m_g = zeros(length(lambdaGvec),1);
m_i = zeros(length(lambdaGvec),1);
minip = zeros(length(lambdaGvec),1);
maxip = zeros(length(lambdaGvec),1);

% Initial guess
ps = tau*snodes(:,2);
ys = zeros(length(snodes),1);
is = rstar*ones(length(snodes),1)+sigma*snodes(:,1)+0.01*snodes(:,2);
gs = zeros(length(snodes),1);

cp = Phi\ps;
cy = Phi\ys;
ci = Phi\is;
cg = Phi\gs;

Ep_p = zeros(length(snodes),1);
Ey_p = zeros(length(snodes),1);
Eg_p = zeros(length(snodes),1);
for k=1:1:length(ws);
    df = rho*snodes(:,1)+ones(length(snodes),1)*epsilons(k,:);
    Ep_p = Ep_p + ws(k)*funeval(cp,fspace,[df ps],[0 1]); % (cp[A(j),b(j+1)]-cp[A(j),b(j)])/h_b
    Ey_p = Ey_p + ws(k)*funeval(cy,fspace,[df ps],[0 1]);
    Eg_p = Eg_p + ws(k)*funeval(cg,fspace,[df ps],[0 1]);
end;

load polfuncoef_tau_0.3.mat;


% Grid of lambdaGs
disp('Number of iterations');
disp(length(lambdaGvec));
disp(' ');
disp('Iteration number:');

for q = 1:length(lambdaGvec);
    
eval(['save Results_lambdaG' int2str(q-1) '.mat Closs loss zlbfreq sd_pi sd_y sd_g sd_i m_pi m_y m_g m_i maxip minip cpvec cyvec cgvec civec Ep_pvec Ey_pvec Eg_pvec;']);
        
lambdaG = lambdaGvec(q);
params.lambdaG = lambdaG;
disp([q lambdaG]); 

Ep_pnew = zeros(length(snodes),1);
Ey_pnew = zeros(length(snodes),1);
Eg_pnew = zeros(length(snodes),1);
Ep_pold = 1000*ones(length(snodes),1);
Ey_pold = 1000*ones(length(snodes),1);
Eg_pold = 1000*ones(length(snodes),1);


while norm([Ep_pnew-Ep_pold;Ey_pnew-Ey_pold;Eg_pnew-Eg_pold],inf) > 1.49*10^(-8);

changepartial = norm([Ep_pnew-Ep_pold;Ey_pnew-Ey_pold],inf);
   
Ep_pold = Ep_p;
Ey_pold = Ey_p;
Eg_pold = Eg_p;
    
stopall = 0;
for it = 1:maxit;
    cpold = cp;
    cyold = cy;
    ciold = ci;
    cgold = cg;
    
    [p,y,i,g] = NKMsolve(snodes,cp,cy,ci,cg,epsilons,ws,fspace,params,Ep_p,Ey_p,Eg_p);
                                    

    % Update policy functions (Note, that I use linear splines)
    cpnew = Phi\p;
    cynew = Phi\y;
    cinew = Phi\i;
    cgnew = Phi\g;
    
    cp = upd*cpnew+(1-upd)*cpold;
    cy = upd*cynew+(1-upd)*cyold;
    ci = upd*cinew+(1-upd)*ciold;
    cg = upd*cgnew+(1-upd)*cgold;
    
    change = norm([cpnew-cpold;cynew-cyold;cinew-ciold;cgnew-cgold],inf);

    if change > 50, disp('Algorithm does not converge.'), stopall = 1; break, end;
    if isnan(change) ==1, disp('Algorithm does not converge.'), stopall = 1; break, end;
    
    if change < tol, break, end;
    
end;

Ep_p = zeros(length(snodes),1);
Ey_p = zeros(length(snodes),1);
Eg_p = zeros(length(snodes),1);

paux = funeval(cp,fspace,snodes);

for k=1:1:length(ws);
    df = rho*snodes(:,1)+ones(length(snodes),1)*epsilons(k,:);
    Ep_p = Ep_p + ws(k)*funeval(cp,fspace,[df paux],[0 1]); % (cp[d(j),i(j+1)]-cp[d(j),i(j)])/h_p
    Ey_p = Ey_p + ws(k)*funeval(cy,fspace,[df paux],[0 1]);
    Eg_p = Eg_p + ws(k)*funeval(cg,fspace,[df paux],[0 1]);
end;

Ep_pnew = Ep_p;
Ey_pnew = Ey_p;
Eg_pnew = Eg_p;

Ep_p = up*Ep_pnew+(1-up)*Ep_pold;    
Ey_p = up*Ey_pnew+(1-up)*Ey_pold;  
Eg_p = up*Eg_pnew+(1-up)*Eg_pold;  
end;


if stopall == 0;
    
cpvec(:,q) = cp;
cyvec(:,q) = cy;
cgvec(:,q) = cg;
civec(:,q) = ci;
Ep_pvec(:,q) = Ep_p;
Ey_pvec(:,q) = Ey_p;
Eg_pvec(:,q) = Eg_p;
    
%% Simulation 
nobs = 2000;
T = 1000; 
buin = 50;

[std_p,std_y,std_i,std_g,mean_p,mean_y,mean_i,mean_g,lobo,simloss,maxima,minima] = ...
NKMsimul(cp,cy,ci,cg,fspace,T,buin,nobs,params);


sd_pi(q) = mean(std_p);
sd_y(q) = mean(std_y);
sd_i(q) = mean(std_i);
sd_g(q) = mean(std_g);
zlbfreq(q) = (mean(lobo))*100;
m_pi(q) = mean(mean_p);
m_y(q) = mean(mean_y);
m_i(q) = mean(mean_i);
m_g(q) = mean(mean_g);
minip(q) = min(minima);
maxip(q) = max(maxima);


loss(q) = mean(simloss);
% Loss in percent of steady state private consumption 
% Remark I: Devide LOSS by 100^(2) because inflation and output gap are
% expressed in percentage terms.
% Remark II: Multiply by 100 because we want to express the loss in
% percentage terms
Closs(q) = (1/2)*(1-beta)*(1/kappa)*theta*(sigma+eta)*loss(q)/(100^(2))*100; 

end;

end;


[Clossmin,pos] = min(Closs);
lambdaGopt = lambdaGvec(pos);

% Save results
eval(['save Results_tau_' num2str(tau) '.mat tau Clossmin lambdaGopt pos lambdaGvec Closs loss zlbfreq sd_pi sd_y sd_g sd_i m_pi m_y m_g m_i maxip minip cpvec cyvec cgvec civec Ep_pvec Ey_pvec Eg_pvec;']);
disp('END');

% Plot results
figure;
plot(lambdaGvec,Closs,'LineWidth',2);
title('Consumption loss');

disp('END');


